{******************************************************************************
Touch - changes the date of a file. Version 1.0.2
By Uri Fridman - urifrid@yahoo.com
freeware under GNU's General Public License (GPL)

Please read the license in gnu-license.txt
Please email the author any application derived from the use of this source code
and/or mention the author's name on any about box, help or acknoledgement.

NO WARRANTY

BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE
STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM
"AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE. THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE
PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY
COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE THE
PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL,
SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY
TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF
THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER
PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
******************************************************************************}
program touch;
{$APPTYPE CONSOLE}
uses
  SysUtils;

function SetFileDate (TheFileName: string; aDate: string):integer;
Var
  f,ret : integer;
Begin
  f := FileOpen (TheFileName, fmOpenReadWrite);
  Try
    ret:=FileSetDate (f, DateTimeToFileDate ((StrToDateTime (aDate))));
  Finally
    FileClose (f);
  End;
  result:=ret;
End;

procedure help;
begin
  writeln('Touch - changes the date of a file. Version 1.0.2');
  writeln('By Uri Fridman, urifrid@yahoo.com');
  writeln('freeware, no warranties, use at your own risk');
  writeln;
  writeln('usage:');
  writeln('touch [-d date] <filename1> [filename2] ...');
  writeln;
  writeln('argument -d date is optional. current time is taken by default');
  writeln('use touch -? or --help to see this help');
  writeln('report bugs to urifrid@yahoo.com');
end;

var
  Found: integer;
  SRec: TSearchRec;
  attributes,newattributes:word;
  i,retval, from:integer;
  dat:string;
begin
  dat:='';
  case paramcount of
  0:begin
      //show help
      help;
      halt(0);
    end;
  1:begin
      //use now as the date to touch file
      if (paramstr(1)='-?') or (paramstr(1)='/?') or (paramstr(1)='--help') then
      begin
        help;
        halt(0);
      end;

      for i:=1 to paramcount do
      begin
        Found:= FindFirst(ParamStr(i),faReadOnly or faSysFile,SRec);
        if Found<> 0 then
        begin
          Writeln('touch: file not found: ',ParamStr(i));
          exit;
        end;// if
        while Found= 0 do
        begin
          try
           attributes:=filegetattr(SRec.Name);
           newattributes:=attributes;
           newattributes:=newattributes and not faReadOnly;
           filesetattr(SRec.Name,newattributes);
          except
           writeln;
           writeln('touch: can''t touch '+SRec.Name+', file might be in use.');
           exit;
          end;

          write('touching '+SRec.Name+'... ');
          dat:=datetimetostr(now);
          retval:=SetFileDate(SRec.Name,dat);
          if retval=0 then
            writeln('OK')
          else
            writeln('FAILED');

          Found:= FindNext(SRec);
        end;// while
        FindClose(SRec);
      end;//for
    end;
  else
    begin
      //use now if no -d param is passed or use date in -d
      if (paramstr(1)='-?') or (paramstr(1)='/?') or (paramstr(1)='--help') then
      begin
        help;
        halt(0);
      end;

      if (paramstr(1)='-d') or (paramstr(1)='-D') then
      begin
        dat:=paramstr(2);
        from:=3;
        try
          strtodatetime(dat);
        except
          writeln('touch: invalid date format: '+dat);
          halt(1);
        end;
      end
      else
      begin
          dat:=datetimetostr(now);
          from:=1;
      end;

      for i:=from to paramcount do
      begin
        Found:= FindFirst(ParamStr(i),faReadOnly or faSysFile,SRec);
        if Found<> 0 then
        begin
          Writeln('touch: file not found: ',ParamStr(i));
          exit;
        end;// if
        while Found= 0 do
        begin
          try
           attributes:=filegetattr(SRec.Name);
           newattributes:=attributes;
           newattributes:=newattributes and not faReadOnly;
           filesetattr(SRec.Name,newattributes);
          except
           writeln;
           writeln('touch: can''t touch '+SRec.Name+', file might be in use.');
           exit;
          end;

          write('touching '+SRec.Name+'... ');

          retval:=SetFileDate(SRec.Name,dat);
          if retval=0 then
            writeln('OK')
          else
            writeln('FAILED');

          Found:= FindNext(SRec);
        end;// while
        FindClose(SRec);
      end;//for
    end;
  end;
end.